{-# OPTIONS --call-by-name #-}

-- This module contains the proof of completeness.

open import Lib.Equality as Eq using (_==_; auto)
open import Lib.Proposition
open import Lib.Word
open import Lib.Inspect
open import Lib.List
open import Lib.Word-Lemmas
open import Lib.Word-Tactics
open import Lib.Reidemeister-Schreier

module Completeness where


open import TwoLevel-Lemmas
open TwoLevel-Rewrite-Twolevel
module TRT2 = TwoLevel-Rewrite-Twolevel2
module TRT3 = TwoLevel-Rewrite-Twolevel3
open Inverse-TwoLevel

open import Gate


import Step1.Theorem as Step1
import Step2.Theorem as Step2
import Step3.Main as Step3
import Step4.Main as Step4
import Step5.Main as Step5
import Step6.Main as Step6
import Step7.Main as Step7
import Step8.Main as Step8


open import Theorem using (module TwoLevel ; module CliffordCS ; completeness-property)
open Step1 using (module TwoLevel-Less)
open Step2 using (module TwoLevel-Simplified)
open Step3 using (Cosets ; I ; I₁)

open import Step1.Completeness renaming (hypA to Step1-hypA ; hypB to Step1-hypB ; g to Step1-g) hiding (completeness)
open import Step2.Completeness renaming (hypA to Step2-hypA ; hypB to Step2-hypB) hiding (completeness)

Gen0 = TwoLevel.Gen
Rel0 = TwoLevel.Rel

Gen1 = TwoLevel-Less.Gen
Rel1 = TwoLevel-Less.Rel

Gen2 = TwoLevel-Simplified.Gen
Rel2 = TwoLevel-Simplified.Rel

import Step3.Rel as R3
import Step4.Rel as R4
import Step5.Rel as R5
import Step6.Rel as R6
import Step7.Rel as R7

Gen3 = Gate.Gate
Rel3 = R3.Rel

Gen4 = Gen3
Rel4 = R4.Rel

Gen5 = Gen3
Rel5 = R5.Rel

Gen6 = Gen3
Rel6 = R6.Rel

Gen7 = Gen3
Rel7 = R7.Rel

Gen8 = CliffordCS.Gen
Rel8 = CliffordCS.Rel

f01 = Step1.f
f10 = Step1-g

f12 = Step2.f
f21 = Step2.g

f23 = Step3.h
f32 = Step3.simple-of-gate-gen

f34 = Step4.simple-of-gen
f43 = Step4.gen-of-simple

f45 = Step5.simple-of-gen
f54 = Step5.gen-of-simple

f56 = Step6.simple-of-gen
f65 = Step6.gen-of-simple

f67 = Step7.simple-of-gen
f76 = Step7.gen-of-simple

f78 = Step8.simple-of-gen
f87 = Step8.gen-of-simple


module S012 = Reidemeister-Schreier-SS {Gen0} {Rel0} {Gen1} {Rel1} {Gen2} {Rel2} f10 f01 f21 f12 Step1-hypA (\ x y -> Step1-hypB {x} {y}) Step2-hypA Step2-hypB

module S013 = Reidemeister-Schreier-FS {Gen0} {Rel0} {Gen2} {Rel2} {Gen3} {Rel3} Cosets I S012.hf S012.gk f23 f32 S012.hypA3 S012.hypB3 Step3.hypA Step3.hypB
module S345 = Reidemeister-Schreier-SS {Gen3} {Rel3} {Gen4} {Rel4} {Gen5} {Rel5} f34 f43 f45 f54 Step4.hypA ( \x y -> Step4.hypB {x} {y}) Step5.hypA ( \ x y -> Step5.hypB {x} {y})
module S567 = Reidemeister-Schreier-SS {Gen5} {Rel5} {Gen6} {Rel6} {Gen7} {Rel7} f56 f65 f67 f76 Step6.hypA ( \ x y -> Step6.hypB {x} {y}) Step7.hypA ( \x y -> Step7.hypB {x} {y})
--module S678 = Reidemeister-Schreier-SS {Gen6} {Rel6} {Gen7} {Rel7} {Gen8} {Rel8} f67 f76 f78 f87 Step7.hypA ( \ x y -> Step7.hypB {x} {y}) Step8.hypA ( \x y -> Step8.hypB {x} {y})
module S357 = Reidemeister-Schreier-SS {Gen3} {Rel3} {Gen5} {Rel5} {Gen7} {R7.Rel} S345.hf S345.gk S567.hf S567.gk S345.hypA3 S345.hypB3 S567.hypA3 S567.hypB3
module S378 = Reidemeister-Schreier-SS {Gen3} {Rel3} {Gen7} {Rel7} {CliffordCS.Gen} {CliffordCS.Rel} S357.hf S357.gk f78 f87 S357.hypA3 S357.hypB3 Step8.hypA (\ x y -> Step8.hypB {x} {y})
module S038 = Reidemeister-Schreier-SF {TwoLevel.Gen} {TwoLevel.Rel} {Gen3} {Rel3} {CliffordCS.Gen} {CliffordCS.Rel} Cosets I S013.hf S013.gk S378.hf S378.gk S013.hypA3 S013.hypB3 S378.hypA3 S378.hypB3

open S038


open import Step2.TwoLevel-Simplified-Lemmas
open Associative
open Monoid-Equational
open import Step1.Soundness
import Step2.Soundness as T2S

module S234 = Reidemeister-Schreier-SF {Gen2} {Rel2} {Gen3} {Rel3} {Gen4} {Rel4} Cosets I f23 f32 f34 f43  Step3.hypA (Step3.hypB) Step4.hypA (\ x y -> Step4.hypB {x} {y})

module S12' = Reidemeister-Schreier-Simplified {Gen2} {Rel2} {Gen1} {Rel1} f12 f21 Step2-hypA Step2-hypB
module S012' = Reidemeister-Schreier-SS {TwoLevel.Gen} {TwoLevel.Rel} {Gen1} {Rel1} {Gen2} {Rel2} f01 f01 f21 f12 Step1-hypA (\ x y -> Step1-hypB {x} {y}) Step2-hypA Step2-hypB

normalize-tlxs : Word TwoLevel.Gen -> Word TwoLevel.Gen
normalize-tlxs w = (S012'.gk *) (word-of-list (TRn.multistep 200 (list-of-word ((S012'.hf *) w))))

lemma-id* : ∀ {x} -> (f01 *) x == x
lemma-id* {gen (TwoLevel.i-gen x)} = _==_.refl
lemma-id* {gen (TwoLevel.X-gen jk)} = _==_.refl
lemma-id* {gen (TwoLevel.K-gen jk)} = _==_.refl
lemma-id* {ε} = _==_.refl
lemma-id* {x • x₁} with lemma-id* {x} | lemma-id* {x₁}
... | h1 | h2 = Eq.cong _•_ h1 h2

open import Index

lemma-aux00 : ∀ {w} -> TwoLevel.Rel ⊢ w === (f12 *) ((f21 *) w)
lemma-aux00 {gen (TwoLevel.i-gen Index.Index.₀)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.i-gen Index.Index.₁)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.i-gen Index.Index.₂)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.i-gen Index.Index.₃)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.i-gen Index.Index.₄)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.i-gen Index.Index.₅)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.i-gen Index.Index.₆)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.i-gen Index.Index.₇)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₀₁)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₀₂)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₀₃)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₀₄)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₀₅)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₀₆)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₀₇)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₁₂)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₁₃)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₁₄)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₁₅)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₁₆)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₁₇)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₂₃)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₂₄)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₂₅)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₂₆)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₂₇)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₃₄)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₃₅)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₃₆)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₃₇)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₄₅)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₄₆)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₄₇)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₅₆)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₅₇)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.X-gen ₆₇)} = TRT2.rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₀₁)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₀₂)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₀₃)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₀₄)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₀₅)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₀₆)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₀₇)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₁₂)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₁₃)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₁₄)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₁₅)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₁₆)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₁₇)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₂₃)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₂₄)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₂₅)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₂₆)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₂₇)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₃₄)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₃₅)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₃₆)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₃₇)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₄₅)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₄₆)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₄₇)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₅₆)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₅₇)} = rewrite-twolevel 100 auto
lemma-aux00 {gen (TwoLevel.K-gen ₆₇)} = rewrite-twolevel 100 auto
lemma-aux00 {ε} = rewrite-twolevel 100 auto
lemma-aux00 {w • w'} with lemma-aux00 {w} | lemma-aux00 {w'}
... | h1 | h2 = cong h1 h2



lemma-normalize-tlxs : ∀ (w : Word TwoLevel.Gen) -> TwoLevel.Rel ⊢ w === normalize-tlxs w
lemma-normalize-tlxs w = symm (
  equational normalize-tlxs w
    by refl
  equals (S012'.gk *) (word-of-list (TRn.multistep 200 (list-of-word ((S012'.hf *) w))))
    by refl' (lemma-*-* f12 f01 ((word-of-list (TRn.multistep 200 (list-of-word ((S012'.hf *) w)))))) reversed
  equals (f01 *) ((f12 *) (word-of-list (TRn.multistep 200 (list-of-word ((S012'.hf *) w)))))
    by soundness (T2S.soundness (TRn.lemma-multistep 200 (list-of-word ((S012'.hf *) w)))) reversed
  equals (f01 *) ((f12 *) ( ( word-of-list (list-of-word ((S012'.hf *) w)))))
    by soundness (T2S.soundness (lemma-list-of-word ((S012'.hf *) w))) reversed
  equals (f01 *) ((f12 *) ( (  ( ((S012'.hf *) w)))))
    by soundness (T2S.soundness (refl' (lemma-*-* f10 f21 w))) reversed
  equals (f01 *) ((f12 *) (  (f21 *) ((f10 *) w)))
    by soundness (T2S.soundness (S12'.lemma-b (refl' (Eq.symm (lemma-id* {w}))))) reversed
  equals (f01 *) ((f12 *) (  (f21 *) ( w)))
    by refl' (lemma-id* {((f12 *) (  (f21 *) ( w)))})
  equals (f12 *) ((f21 *) w)
    by lemma-aux00 {w} reversed
  equals w
  )

open Rewriting
open import CosetNF

module BC2 = Basis-Change-With-Standardization TwoLevel-Lemmas.group-like (TwoLevel-Step-Order.step then TwoLevel-Step-Twolevel.step then TwoLevel-Step-Twolevel.X-step) Commuting-TwoLevel.comm-canonical Commuting-TwoLevel.lemma-comm-canonical

module BC4 = Basis-Change-With-Standardization TwoLevel-Lemmas.group-like (TwoLevel-Step-Twolevel.step) (Legacy.listf-of-f normalize-tlxs) (Legacy.lemma-listf-of-f lemma-normalize-tlxs)
module TRT4 = Step-With-Standardization (step-cong (TwoLevel-Step-Order.step then TwoLevel-Step-Twolevel.step)) (Legacy.listf-of-f normalize-tlxs) (Legacy.lemma-listf-of-f lemma-normalize-tlxs) renaming (general-rewrite to rewrite-twolevel)

open Commuting-TwoLevel

lemma-f : ∀ (x : CliffordCS.Gen) -> TwoLevel.Rel ⊢ Theorem.f x === S038.gk x
lemma-f CliffordCS.S0-gen = rewrite-twolevel 100 auto
lemma-f CliffordCS.S1-gen = rewrite-twolevel 100 auto
lemma-f CliffordCS.S2-gen = rewrite-twolevel 100 auto
lemma-f CliffordCS.CS01-gen = rewrite-twolevel 100 auto
lemma-f CliffordCS.CS12-gen = rewrite-twolevel 100 auto
lemma-f CliffordCS.iI-gen = rewrite-twolevel 100 auto
lemma-f CliffordCS.K0-gen = rewrite-twolevel 100 auto
lemma-f CliffordCS.K2-gen = aux
  where
    open TwoLevel
    
    aux :  Rel ⊢ K ₀₁ • K ₂₃ • K ₄₅ • K ₆₇ === gk CliffordCS.K2-gen
    aux =
      equational K ₀₁ • K ₂₃ • K ₄₅ • K ₆₇
        by rewrite-twolevel 50 auto
      equals (X ₃₆ • X ₁₄) • (K ₀₄ • K ₁₅ • K ₂₆ • K ₃₇) • X ₃₆ • X ₁₄
        by right left lemma-f CliffordCS.K0-gen
      equals (X ₃₆ • X ₁₄) • (gk CliffordCS.K0-gen) • X ₃₆ • X ₁₄
        by right-unit reversed
      equals ((X ₃₆ • X ₁₄) • (gk CliffordCS.K0-gen) • X ₃₆ • X ₁₄) • ε
        by right lemma-normalize-tlxs (X ₁₂ • X ₃₄ • X ₂₃ • X ₃₄ • X ₅₆ • X ₄₅ • X ₃₄ • X ₄₅ • X ₃₄ • X ₂₃ • X ₁₂ • X ₃₄ • X ₄₅ • X ₃₄ • X ₄₅ • X ₅₆) reversed
      equals ((X ₃₆ • X ₁₄) • (gk CliffordCS.K0-gen) • X ₃₆ • X ₁₄) • X ₁₂ • X ₃₄ • X ₂₃ • X ₃₄ • X ₅₆ • X ₄₅ • X ₃₄ • X ₄₅ • X ₃₄ • X ₂₃ • X ₁₂ • X ₃₄ • X ₄₅ • X ₃₄ • X ₄₅ • X ₅₆
        by rewrite-twolevel 100 auto
      equals gk CliffordCS.K2-gen
lemma-f CliffordCS.K1-gen = aux2
  where
    open TwoLevel

    aux2 :  Rel ⊢ K ₀₂ • K ₁₃ • K ₄₆ • K ₅₇ === gk CliffordCS.K1-gen
    aux2 =
      equational K ₀₂ • K ₁₃ • K ₄₆ • K ₅₇
        by right-unit reversed
      equals (K ₀₂ • K ₁₃ • K ₄₆ • K ₅₇) • ε
        by right lemma-normalize-tlxs (X ₃₄ • X ₂₃ • X ₃₄ • X ₄₅ • X ₃₄ • X ₄₅ • X ₃₄ • X ₂₃ • X ₃₄ • X ₄₅ • X ₃₄ • X ₄₅) reversed
      equals (K ₀₂ • K ₁₃ • K ₄₆ • K ₅₇) • (X ₃₄ • X ₂₃ • X ₃₄ • X ₄₅ • X ₃₄ • X ₄₅ • X ₃₄ • X ₂₃ • X ₃₄ • X ₄₅ • X ₃₄ • X ₄₅)
        by rewrite-twolevel 100 auto
      equals gk CliffordCS.K1-gen


lemma-f* : ∀ (x : Word CliffordCS.Gen) -> TwoLevel.Rel ⊢ (Theorem.f *) x === (S038.gk *) x
lemma-f* (gen x) = lemma-f x
lemma-f* ε = refl
lemma-f* (w • w') with lemma-f* w | lemma-f* w'
... | h1 | h2 = cong h1 h2

completeness : completeness-property
completeness {w} {v} hyp = S038.reidemeister-schreier w v hyp'
  where
    hyp' : TwoLevel.Rel ⊢ (S038.gk *) w === (S038.gk *) v
    hyp' =
      equational (S038.gk *) w
        by lemma-f* w reversed
      equals (Theorem.f *) w
        by hyp
      equals (Theorem.f *) v
        by lemma-f* v
      equals (S038.gk *) v

